@@ -1,8 +1,7 @@
-# dbusiface.py
 #
 # Author: Facundo Batista <facundo@taniquetil.com.ar>
 #
-# Copyright 2010 Chicharreros
+# Copyright 2010-2011 Chicharreros
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -20,31 +19,20 @@
 
 import collections
 import logging
-import re
 
 import dbus
 from dbus import SessionBus
 from dbus.mainloop.glib import DBusGMainLoop
 from twisted.internet import defer
 
-try:
-    # yes, they can be imported! pylint: disable=F0401,E0611
-    from ubuntuone.platform.tools import SyncDaemonTool, DBusClient
-    from ubuntuone.platform.dbus_interface import DBUS_IFACE_PUBLIC_FILES_NAME
-except ImportError:
-    # support for old structure (pre-Natty)
-    # yes, they can be imported! pylint: disable=F0401,E0611
-    from ubuntuone.syncdaemon.tools import SyncDaemonTool, DBusClient
-    from ubuntuone.syncdaemon.dbus_interface import (
-        DBUS_IFACE_PUBLIC_FILES_NAME,
-    )
+# yes, they can be imported! pylint: disable=F0401,E0611
+from ubuntuone.platform.tools import SyncDaemonTool, is_already_running
 
 # log!
 logger = logging.getLogger('magicicada.dbusiface')
 
 # we use here camel case names, because this variables are used later as
 # classes, so pylint: disable=C0103
-QueueData = collections.namedtuple('QueueData', 'operation path share node')
 FolderData = collections.namedtuple('FolderData',
                                 'node path suggested_path subscribed volume')
 ShareData = collections.namedtuple('ShareData', 'accepted access_level '
@@ -53,33 +41,33 @@ ShareData = collections.namedtuple('ShareData', 'accepted access_level '
 PublicFilesData = collections.namedtuple('PublicFilesData',
                                          'volume node path public_url')
 
-# regular expressions for parsing MetaQueue data
-RE_OP_LISTDIR = re.compile("(ListDir)\(share_id=(.*?), node_id=(.*?), .*")
-RE_OP_UNLINK = re.compile("(Unlink)\(share_id=(.*?), node_id=(.*?), .*")
-RE_OP_MAKEFILE = re.compile(
-                "(MakeFile)\(share_id=(.*?), parent_id=(.*?), name=(.*?), .*")
-RE_OP_MAKEDIR = re.compile(
-                "(MakeDir)\(share_id=(.*?), parent_id=(.*?), name=(.*?), .*")
-RE_OP_MOVE = re.compile(
-                "(Move)\(share_id=(.*?), node_id=(.*?), old_parent_id=(.*?), "
-                "new_parent_id=(.*?), new_name=(.*?)\)")
-
 # DBus exceptions store the type inside, as a string :|
 DBUSERR_NOREPLY = 'org.freedesktop.DBus.Error.NoReply'
-DBUSERR_NAMENOOWNER = 'org.freedesktop.DBus.Error.NameHasNoOwner'
 DBUSERR_PYKEYERROR = 'org.freedesktop.DBus.Python.KeyError'
 
 # some constants
 NOT_SYNCHED_PATH = "Not a valid path!"
 
 
-# some exceptions
 class ShareOperationError(Exception):
     """Error on an operation on a share."""
-    def __init__(self, share_id, error):
+    def __init__(self, share_id=None, error=None):
         self.share_id = share_id
         self.error = error
-        Exception.__init__(self)
+        super(ShareOperationError, self).__init__(error)
+
+
+class FolderOperationError(Exception):
+    """Error on an operation on a folder.
+
+    The 'path' will be present on a CreateFolder error, in the rest of
+    operations the volume_id will be the identificator.
+    """
+    def __init__(self, path=None, volume_id=None, error=None):
+        self.path = path
+        self.volume_id = volume_id
+        self.error = error
+        super(FolderOperationError, self).__init__(error)
 
 
 def _is_retry_exception(err):
@@ -128,9 +116,8 @@ class DBusInterface(object):
         # hook up for signals and store info for the shutdown
         _signals = [
             (self._on_status_changed, 'Status', 'StatusChanged'),
-            (self._on_content_queue_changed, 'Status', 'ContentQueueChanged'),
-            (self._on_meta_queue_changed, 'Status', 'MetaQueueChanged'),
-            (self._on_name_owner_changed, None, 'NameOwnerChanged'),
+            (self._on_queue_added, 'Status', 'RequestQueueAdded'),
+            (self._on_queue_removed, 'Status', 'RequestQueueRemoved'),
             (self._on_folder_created, 'Folders', 'FolderCreated'),
             (self._on_folder_deleted, 'Folders', 'FolderDeleted'),
             (self._on_folder_subscribed, 'Folders', 'FolderSubscribed'),
@@ -138,7 +125,6 @@ class DBusInterface(object):
             (self._on_share_created, 'Shares', 'ShareCreated'),
             (self._on_share_deleted, 'Shares', 'ShareDeleted'),
             (self._on_share_changed, 'Shares', 'ShareChanged'),
-            (self._on_public_files_list, 'PublicFiles', 'PublicFilesList'),
             (self._on_public_files_changed, 'PublicFiles',
                                             'PublicAccessChanged'),
         ]
@@ -182,6 +168,54 @@ class DBusInterface(object):
         d.addCallback(self._process_status)
         return d
 
+    @retryable
+    @defer.inlineCallbacks
+    def get_free_space(self, volume_id):
+        """Get the free space for a volume."""
+        result = yield self.sync_daemon_tool.free_space(volume_id)
+        logger.info("Free space for volume %r is %r", volume_id, result)
+        defer.returnValue(result)
+
+    @retryable
+    @defer.inlineCallbacks
+    def get_real_shares_dir(self):
+        """Get the real directory for the shares."""
+        result = yield self.sync_daemon_tool.get_shares_dir()
+        logger.info("Real shares dir: %r", result)
+        defer.returnValue(result)
+
+    @retryable
+    @defer.inlineCallbacks
+    def get_link_shares_dir(self):
+        """Get the link directory for the shares."""
+        result = yield self.sync_daemon_tool.get_shares_dir_link()
+        logger.info("Link shares dir: %r", result)
+        defer.returnValue(result)
+
+    def _process_transfers(self, transfers, progress):
+        """Process downloads or uploads to keep useful info only."""
+        r = [(t['path'], int(t[progress]), int(t['deflated_size']))
+             for t in transfers if 'deflated_size' in t]
+        return r
+
+    @retryable
+    @defer.inlineCallbacks
+    def get_current_downloads(self):
+        """Get the current_downloads."""
+        result = yield self.sync_daemon_tool.get_current_downloads()
+        processed = self._process_transfers(result, 'n_bytes_read')
+        logger.info("Get current downloads: %d items", len(processed))
+        defer.returnValue(processed)
+
+    @retryable
+    @defer.inlineCallbacks
+    def get_current_uploads(self):
+        """Get the current_uploads."""
+        result = yield self.sync_daemon_tool.get_current_uploads()
+        processed = self._process_transfers(result, 'n_bytes_written')
+        logger.info("Get current uploads: %d items", len(processed))
+        defer.returnValue(processed)
+
     def _on_status_changed(self, state):
         """Call the SD callback."""
         logger.info("Received Status changed")
@@ -189,29 +223,17 @@ class DBusInterface(object):
         data = self._process_status(state)
         self.msd.on_sd_status_changed(*data)
 
-    def _on_content_queue_changed(self, _=None):
+    def _on_queue_added(self, op_name, op_id, op_data):
         """Call the SD callback."""
-        logger.info("Received Content Queue changed")
-        self.msd.on_sd_content_queue_changed()
+        logger.debug("Received Queue added: %r [%s] %s",
+                     op_name, op_id, op_data)
+        self.msd.on_sd_queue_added(op_name, op_id, op_data)
 
-    def _on_meta_queue_changed(self):
+    def _on_queue_removed(self, op_name, op_id, op_data):
         """Call the SD callback."""
-        logger.info("Received Meta Queue changed")
-        self.msd.on_sd_meta_queue_changed()
-
-    def _on_name_owner_changed(self, name, oldowner, newowner):
-        """Receive the NameOwnerChanged signal from DBus."""
-        if name != 'com.ubuntuone.SyncDaemon':
-            return
-
-        logger.info("Received Name Owner changed")
-        logger.debug("Name Owner data: %r %r", oldowner, newowner)
-        old = bool(oldowner)
-        new = bool(newowner)
-        if old == new:
-            logger.error("Name Owner invalid data: Same bool in old and new!")
-            return
-        self.msd.on_sd_name_owner_changed(new)
+        logger.debug("Received Queue removed: %r [%s] %s",
+                     op_name, op_id, op_data)
+        self.msd.on_sd_queue_removed(op_name, op_id, op_data)
 
     def _on_folder_created(self, _):
         """Call the SD callback."""
@@ -266,16 +288,30 @@ class DBusInterface(object):
                                 path=d['path'], public_url=d['public_url'])
             processed.append(p)
 
-        # trigger the deferred if have one, else call the msd with the data
-        if self._public_files_deferred is None:
-            self.msd.on_sd_public_files_list(processed)
-        else:
-            d = self._public_files_deferred
-            self._public_files_deferred = None
-            d.callback(processed)
+        return processed
 
+    @defer.inlineCallbacks
     def get_public_files(self):
         """Ask the Public Files info to syncdaemon."""
+        try:
+            result = yield self.sync_daemon_tool.get_public_files()
+            logger.debug("Public files asked ok.")
+        except AttributeError:
+            logger.warning('Method sdtool.get_public_files is not available, '
+                           'trying old one directly from dbus.')
+            result = yield self.get_public_files_old()
+        except:
+            logger.exception("Public files finished with error:")
+            result = []
+
+        defer.returnValue(self._on_public_files_list(result))
+
+    def get_public_files_old(self):
+        """Ask the Public Files info to syncdaemon (old approach)."""
+        # yes, they can be imported! pylint: disable=F0401,E0611,W0404
+        from ubuntuone.platform.tools import DBusClient, ErrorSignal
+        from ubuntuone.platform.dbus_interface import \
+            DBUS_IFACE_PUBLIC_FILES_NAME
         client = DBusClient(self._bus, '/publicfiles',
                             DBUS_IFACE_PUBLIC_FILES_NAME)
 
@@ -291,136 +327,19 @@ class DBusInterface(object):
             """Call was not succesful."""
             logger.error("Public files asked with error: %s", error)
 
+        d = self.sync_daemon_tool.wait_for_signal('PublicFilesList',
+                                                  filter=lambda _: True)
+
         client.call_method('get_public_files',
-                           reply_handler=call_done, error_handler=call_error)
-
-        # return a stored deferred that will be triggered later with the signal
-        d = defer.Deferred()
-        self._public_files_deferred = d
+                           reply_handler=call_done,
+                           error_handler=call_error)
         return d
 
     @retryable
-    def get_content_queue(self):
-        """Get the content queue from SDT."""
-
-        def process(data):
-            """Enhance data format."""
-            logger.info("Processing Content Queue items (%d)", len(data))
-            all_items = []
-            for d in data:
-                logger.debug("    Content Queue data: %r", d)
-                cq = QueueData(operation=d['operation'], path=d['path'],
-                               node=d['node'], share=d['share'])
-                all_items.append(cq)
-            return all_items
-
-        logger.info("Getting content queue")
-        d = self.sync_daemon_tool.waiting_content()
-        d.addCallback(process)
-        return d
-
-    def _parse_mq_old(self, data):
-        """Parse MetaQueue string to extract its data."""
-        if data in ('AccountInquiry', 'FreeSpaceInquiry', 'GetPublicFiles',
-                    'ListShares', 'ListVolumes', 'Query',
-                    'ChangePublicAccess', 'AnswerShare'):
-            return QueueData(operation=data, path=None, node=None, share=None)
-
-        m = RE_OP_LISTDIR.match(data)
-        if m:
-            op, share, node = m.groups()
-            path = '?'  # we should get the real path, no API now
-            return QueueData(operation=op, path=path, node=node, share=share)
-
-        m = RE_OP_MAKEFILE.match(data)
-        if m:
-            op, share, parent, name = m.groups()
-            path = '/?.../' + name  # we should get the real path, no API now
-            return QueueData(operation=op, path=path, node=None, share=share)
-
-        m = RE_OP_MAKEDIR.match(data)
-        if m:
-            op, share, parent, name = m.groups()
-            path = '/?.../' + name  # we should get the real path, no API now
-            return QueueData(operation=op, path=path, node=None, share=share)
-
-        m = RE_OP_UNLINK.match(data)
-        if m:
-            op, share, node, = m.groups()
-            path = '?'  # we should get the real path, no API now
-            return QueueData(operation=op, path=path, node=node, share=share)
-
-        m = RE_OP_MOVE.match(data)
-        if m:
-            op, share, node, old_parent, new_parent, new_name, = m.groups()
-
-            # we should get the real info, no API now
-            old_path = '/?...'
-            old_name = '?'
-            new_path = '/?...'
-            composed_path = "%s/%s -> %s/%s" % (old_path, old_name,
-                                                new_path, new_name)
-            return QueueData(operation=op, path=composed_path,
-                             node=node, share=share)
-
-        raise ValueError("Not supported MetaQueue data: %r" % data)
-
-    def _parse_mq_dict(self, (op, data)):
-        """Use MetaQueue dictionary and prepare command data."""
-        if op in ('AccountInquiry', 'FreeSpaceInquiry', 'Query', 'ListShares',
-                  'GetPublicFiles', 'ListVolumes', 'ChangePublicAccess',
-                  'AnswerShare', 'GetDelta'):
-            return QueueData(operation=op, path=None, node=None, share=None)
-
-        if op in ('ListDir', 'Unlink'):
-            share = data['share_id']
-            node = data['node_id']
-            path = data.get('path', '?')
-            return QueueData(operation=op, path=path, node=node, share=share)
-
-        if op in ('MakeFile', 'MakeDir'):
-            share = data['share_id']
-            name = data['name']
-            path = '/?.../' + name  # we should get the real path, no API now
-            return QueueData(operation=op, path=path, node=None, share=share)
-
-        if op == 'Move':
-            share = data['share_id']
-            node = data['node_id']
-            new_name = data['new_name']
-            old_path = data.get('path', '/?.../?')
-
-            # we should get the real info, no API now
-            new_path = '/?...'
-            composed_path = "%s -> %s/%s" % (old_path, new_path, new_name)
-            return QueueData(operation=op, path=composed_path,
-                             node=node, share=share)
-
-        raise ValueError("Not supported MetaQueue data: %r" % data)
-
-    @retryable
-    def get_meta_queue(self):
-        """Get the meta queue from SDT."""
-
-        def process(data):
-            """Enhance data format."""
-            logger.info("Processing Meta Queue items (%d)", len(data))
-            all_items = []
-            for d in data:
-                logger.debug("    Meta Queue data: %r", d)
-                if isinstance(d, basestring):
-                    # old syncdaemon, sending operations as strings
-                    parsed = self._parse_mq_old(d)
-                else:
-                    # new syncdaemon, dictionary with all the info
-                    parsed = self._parse_mq_dict(d)
-                all_items.append(parsed)
-            return all_items
-
-        logger.info("Getting meta queue")
-        d = self.sync_daemon_tool.waiting_metadata()
-        d.addCallback(process)
-        return d
+    def get_queue_content(self):
+        """Get the queue content from SDT."""
+        logger.info("Getting queue content")
+        return self.sync_daemon_tool.waiting()
 
     @retryable
     def get_folders(self):
@@ -432,10 +351,7 @@ class DBusInterface(object):
             all_items = []
             for d in data:
                 logger.debug("    Folders data: %r", d)
-                f = FolderData(node=d['node_id'], path=d['path'],
-                               suggested_path=d['suggested_path'],
-                               volume=d['volume_id'],
-                               subscribed=bool(d['subscribed']))
+                f = self._get_folder_data(d)
                 all_items.append(f)
             return all_items
 
@@ -464,18 +380,12 @@ class DBusInterface(object):
         logger.info("Calling disconnect")
         self.sync_daemon_tool.disconnect()
 
+    @defer.inlineCallbacks
     def is_sd_started(self):
         """Find out if SD is active in the system."""
-        try:
-            self._bus.get_name_owner('com.ubuntuone.SyncDaemon')
-        except dbus.exceptions.DBusException, err:
-            if err.get_dbus_name() != DBUSERR_NAMENOOWNER:
-                raise
-            started = False
-        else:
-            started = True
+        started = yield is_already_running()
         logger.info("Checking if SD is started: %s", started)
-        return started
+        defer.returnValue(started)
 
     def _process_share_info(self, data):
         """Process share data."""
@@ -552,16 +462,132 @@ class DBusInterface(object):
 
     @retryable
     @defer.inlineCallbacks
-    def accept_share(self, share_id):
-        """Accept a share."""
-        logger.debug("Accept share %s started", share_id)
+    def _answer_share(self, share_id, method, action_name):
+        """Effectively accept or reject a share."""
+        logger.debug("%s share %s started", action_name, share_id)
         try:
-            result = yield self.sync_daemon_tool.accept_share(share_id)
-        except dbus.exceptions.DBusException, e:
-            error = str(e.args[0])
-            logger.debug("Accept share %s crashed: %s", share_id, error)
+            result = yield method(share_id)
+        except Exception, e:
+            if len(e.args) == 2 and len(e.args[1]) > 1:
+                error = "%s (%s)" % (e.args[0], e.args[1][1])
+            else:
+                error = str(e.args[0])
+            logger.debug("%s share %s crashed: %s",
+                         action_name, share_id, error)
             raise ShareOperationError(share_id=share_id, error=error)
 
-        logger.debug("Accept share %s finished: %s", share_id, result)
+        logger.debug("%s share %s finished: %s", action_name, share_id, result)
         if 'error' in result:
             raise ShareOperationError(share_id=share_id, error=result['error'])
+
+    def accept_share(self, share_id):
+        """Accept a share."""
+        return self._answer_share(share_id,
+                                  self.sync_daemon_tool.accept_share, "Accept")
+
+    def reject_share(self, share_id):
+        """Reject a share."""
+        return self._answer_share(share_id,
+                                  self.sync_daemon_tool.reject_share, "Reject")
+
+    def subscribe_share(self, share_id):
+        """Subscribe a share."""
+        args = share_id, self.sync_daemon_tool.subscribe_share, "Subscribe"
+        return self._answer_share(*args)
+
+    def unsubscribe_share(self, share_id):
+        """Unsubscribe a share."""
+        args = share_id, self.sync_daemon_tool.unsubscribe_share, "Unsubscribe"
+        return self._answer_share(*args)
+
+    @retryable
+    @defer.inlineCallbacks
+    def send_share_invitation(self, path, mail_address,
+                              share_name, access_level):
+        """Send a share invitation."""
+        logger.debug("Send share invitation started: path=%r mail_address=%s "
+                     "share_name=%r access_level=%s", path, mail_address,
+                     share_name, access_level)
+        try:
+            yield self.sync_daemon_tool.send_share_invitation(
+                                path, mail_address, share_name, access_level)
+        except dbus.exceptions.DBusException, e:
+            error = str(e.args[0])
+            logger.debug("Send share invitation crashed: %s (path=%r "
+                         "mail_address=%s share_name=%r access_level=%s)",
+                         error, path, mail_address, share_name, access_level)
+            raise ShareOperationError(error=error)
+        else:
+            logger.debug("Send share invitation finished ok (path=%r "
+                         "mail_address=%s share_name=%r access_level=%s)",
+                         path, mail_address, share_name, access_level)
+
+    def _get_folder_data(self, data):
+        """Transform a syncdaemontool folder response in an internal object."""
+        f = FolderData(path=data['path'], subscribed=bool(data['subscribed']),
+                       node=data['node_id'], volume=data['volume_id'],
+                       suggested_path=data['suggested_path'])
+        return f
+
+    @retryable
+    @defer.inlineCallbacks
+    def _folder_operation(self, act_name, action, val_name, value):
+        """Generic folder operation."""
+        logger.debug("%s folder started on %s %r", act_name, val_name, value)
+        try:
+            result = yield action(value)
+        except Exception, e:
+            logger.debug("%s folder finished with error %s on %s %r",
+                         act_name, e, val_name, value)
+            if len(e.args) == 2 and len(e.args[1]) > 1:
+                error = "%s (%s)" % (e.args[0], e.args[1][1])
+            else:
+                error = str(e.args[0])
+            kwargs = {'error': error, val_name: value}
+            raise FolderOperationError(**kwargs)
+        else:
+            logger.debug("%s folder finished ok: %s", act_name, result)
+            folder = self._get_folder_data(result)
+            defer.returnValue(folder)
+
+    def create_folder(self, path):
+        """Create a folder."""
+        args = 'Create', self.sync_daemon_tool.create_folder, 'path', path
+        return self._folder_operation(*args)
+
+    def delete_folder(self, volume_id):
+        """Delete a folder."""
+        args = ('Delete', self.sync_daemon_tool.delete_folder,
+                'volume_id', volume_id)
+        return self._folder_operation(*args)
+
+    def subscribe_folder(self, volume_id):
+        """Subscribe a folder."""
+        args = ('Subscribe', self.sync_daemon_tool.subscribe_folder,
+                'volume_id', volume_id)
+        return self._folder_operation(*args)
+
+    def unsubscribe_folder(self, volume_id):
+        """Unsubscribe a folder."""
+        args = ('Unsubscribe', self.sync_daemon_tool.unsubscribe_folder,
+                'volume_id', volume_id)
+        return self._folder_operation(*args)
+
+    @retryable
+    @defer.inlineCallbacks
+    def change_public_access(self, path, is_public):
+        """Make a file public or not."""
+        logger.debug("Change public access started: path=%r, is_public=%s",
+                     path, is_public)
+        try:
+            res = yield self.sync_daemon_tool.change_public_access(path,
+                                                                   is_public)
+        except Exception, e:
+            logger.debug("Change public access finished with error %r on %r",
+                         e, path)
+            raise
+
+        logger.debug("Change public access finished ok: %s", res)
+        pf = PublicFilesData(volume=res['share_id'], node=res['node_id'],
+                             path=res['path'], public_url=res['public_url'])
+        defer.returnValue(pf)
