@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Natalia Bidart <natalia.bidart@gmail.com>
+# Authors: Natalia Bidart <nataliabidart@gmail.com>
+#          Facundo Batista <facundo@taniquetil.com.ar>
 #
-# Copyright 2010 Chicharreros
+# Copyright 2010-2011 Chicharreros
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -21,35 +22,13 @@
 from __future__ import division
 
 import logging
-import os
 
 from functools import wraps
 
-import gtk
-
-from magicicada.magicicadaconfig import get_data_file
-
 
 NO_OP = lambda *a, **kw: None
 
 
-def get_builder(builder_file_name):
-    """Return a fully-instantiated gtk.Builder instance from specified ui file.
-
-    :param builder_file_name: The name of the builder file, without extension.
-        Assumed to be in the 'ui' directory under the data path.
-    """
-    # Look for the ui file that describes the user interface.
-    ui_filename = get_data_file('ui', builder_file_name)
-    if not os.path.exists(ui_filename):
-        ui_filename = None
-
-    builder = gtk.Builder()
-    builder.set_translation_domain('magicicada')
-    builder.add_from_file(ui_filename)
-    return builder
-
-
 def log(logger, level=logging.DEBUG):
     """Log input/ouput info for 'f' using 'logger'."""
 
@@ -66,7 +45,9 @@ def log(logger, level=logging.DEBUG):
             try:
                 result = f(*args, **kwargs)
             except Exception:  # pylint: disable=W0703
-                logger.exception('%s failed with exception:', name)
+                logger.error('Exception when calling %r with %s %s:',
+                             name, args, kwargs)
+                raise
             logger.log(level, "Returning from '%s' with result '%s'.",
                        name, result)
             return result
