@@ -65,28 +65,37 @@ def exception_handler(exc_type, exc_value, tb):
     logger.error("Unhandled exception!\n%s", msg)
 
 
-def set_up():
+def get_filename(cache_dir=None):
+    """Return the log file name."""
+    if cache_dir is None:
+        # choose the folder to store the logs
+        cache_dir = xdg.BaseDirectory.xdg_cache_home
+
+    return os.path.join(cache_dir, 'magicicada', 'magicicada.log')
+
+
+def set_up(cache_dir=None):
     """Set up the logging."""
-    # choose the folder to store the logs
-    cache = xdg.BaseDirectory.xdg_cache_home
-    logfolder = os.path.join(cache, 'magicicada')
+    logfile = get_filename(cache_dir)
+    logfolder = os.path.dirname(logfile)
     if not os.path.exists(logfolder):
         os.makedirs(logfolder)
-    logfile = os.path.join(logfolder, 'magicicada.log')
 
     logger = logging.getLogger('magicicada')
     handler = CustomRotatingFH(logfile, maxBytes=1e6, backupCount=10)
     logger.addHandler(handler)
-    formatter = logging.Formatter("%(asctime)s  %(name)-23s"
-                                  "%(levelname)-8s %(message)s")
+    assert len(logger.handlers) == 1
+    formatter = logging.Formatter("%(asctime)s - %(name)s - "
+                                  "%(levelname)s - %(message)s")
     handler.setFormatter(formatter)
-    logger.setLevel(logging.INFO)
+    logger.setLevel(logging.DEBUG)
 
     if os.getenv('DEBUG'):
         console = logging.StreamHandler()
         console.setLevel(logging.DEBUG)
         console.setFormatter(formatter)
         logger.addHandler(console)
+        assert len(logger.handlers) == 2
         logger.setLevel(logging.DEBUG)
 
     # hook the exception handler
