@@ -20,20 +20,36 @@
 
 import cStringIO
 import logging
+import os
 import sys
 import unittest
 
 from twisted.python import log, failure
+from ubuntuone.devtools.handlers import MementoHandler
+from xdg.BaseDirectory import xdg_cache_home
 
-from magicicada.logger import exception_handler, deferror_handler
-from magicicada.tests.helpers import MementoHandler
+from magicicada.logger import exception_handler, deferror_handler, get_filename
 
 
 # It's ok to access private data in the test suite
 # pylint: disable=W0212
 
 
-class ExceptionTests(unittest.TestCase):
+class SimpleTestCase(unittest.TestCase):
+    """Several simple tests for the logger module."""
+
+    def test_log_fname_default(self):
+        """The log filename by default."""
+        fname = os.path.join(xdg_cache_home, 'magicicada', 'magicicada.log')
+        self.assertEqual(get_filename(), fname)
+
+    def test_log_fname_basedir(self):
+        """The log filename using other base dir."""
+        fname = os.path.join('tmp', 'magicicada', 'magicicada.log')
+        self.assertEqual(get_filename(cache_dir='tmp'), fname)
+
+
+class ExceptionTestCase(unittest.TestCase):
     """Test that we log on unhandled exceptions."""
 
     def _get_exception_data(self):
@@ -56,6 +72,7 @@ class ExceptionTests(unittest.TestCase):
 
         # call
         l.addHandler(handler)
+        self.addCleanup(lambda: l.removeHandler(handler))
         exc = self._get_exception_data()
         try:
             exception_handler(*exc)
@@ -85,7 +102,7 @@ class ExceptionTests(unittest.TestCase):
         self.assertTrue("ZeroDivisionError" in shown)
 
 
-class DeferredTests(unittest.TestCase):
+class DeferredTestCase(unittest.TestCase):
     """Error logging when it happened inside deferreds."""
 
     def test_observer_added(self):
